import { Server } from 'socket.io'
import { TSocket } from '../../utilities/types'
import ZoneRepository from '../../repositories/zoneRepository'
import ZoneManager from '../../managers/zoneManager'
import { Character, Zone } from '@prisma/client'
import CharacterManager from '../../managers/characterManager'

/**
 * Handle character zone leave event
 * @param socket
 * @param io
 */
export default function (socket: TSocket, io: Server) {
  socket.on('zone:character:leave', async () => {
    if (!socket.character) {
      return
    }

    if (!socket.character.zoneId) {
      return
    }

    const zone = await ZoneRepository.getById(socket.character.zoneId)

    if (!zone) {
      return
    }

    socket.leave(zone.id.toString())

    // let other clients know of new character
    io.to(zone.id.toString()).emit('zone:character:leave', socket.character)

    // add character to zone manager
    await CharacterManager.removeCharacter(socket.character)
  })
}

/**
 * Resources:
 * https://stackoverflow.com/questions/6873607/socket-io-rooms-difference-between-broadcast-to-and-sockets-in
 */
